@@ -1,8 +1,8 @@
 package Mojo::DOM;
-use Mojo::Base -base;
+use Mojo::Base -strict;
 use overload
-  '%{}'    => sub { shift->attrs },
-  'bool'   => sub {1},
+  '%{}'    => sub { shift->attr },
+  bool     => sub {1},
   '""'     => sub { shift->to_xml },
   fallback => 1;
 
@@ -36,24 +36,26 @@ sub new {
   return @_ ? $self->parse(@_) : $self;
 }
 
-sub all_text {
-  my ($self, $trim) = @_;
-  my $tree = $self->tree;
-  return _text(_elements($tree), 1, _trim($tree, $trim));
-}
+sub all_text { shift->_content(1, @_) }
+
+sub ancestors { _select($_[0]->_collect(_ancestors($_[0]->tree)), $_[1]) }
 
 sub append { shift->_add(1, @_) }
 
 sub append_content {
   my ($self, $new) = @_;
   my $tree = $self->tree;
-  push @$tree, @{_parent($self->_parse("$new"), $tree)};
+  push @$tree, _link($self->_parse("$new"), $tree);
   return $self;
 }
 
-sub at { shift->find(@_)->[0] }
+sub at {
+  my $self = shift;
+  return undef unless my $result = $self->_css->select_one(@_);
+  return $self->new->tree($result)->xml($self->xml);
+}
 
-sub attrs {
+sub attr {
   my $self = shift;
 
   # Hash
@@ -70,56 +72,30 @@ sub attrs {
   return $self;
 }
 
-sub charset { shift->_html(charset => @_) }
-
 sub children {
-  my ($self, $type) = @_;
-
-  my @children;
-  my $charset = $self->charset;
-  my $xml     = $self->xml;
-  my $tree    = $self->tree;
-  for my $e (@$tree[($tree->[0] eq 'root' ? 1 : 4) .. $#$tree]) {
-
-    # Make sure child is the right type
-    next unless $e->[0] eq 'tag';
-    next if defined $type && $e->[1] ne $type;
-    push @children, $self->new->charset($charset)->tree($e)->xml($xml);
-  }
-
-  return Mojo::Collection->new(@children);
+  my $self = shift;
+  return _select(
+    $self->_collect(grep { $_->[0] eq 'tag' } _nodes($self->tree)), @_);
 }
 
 sub content_xml {
   my $self = shift;
-
-  # Render children
-  my $tree    = $self->tree;
-  my $charset = $self->charset;
-  my $xml     = $self->xml;
-  return join '', map {
-    Mojo::DOM::HTML->new(charset => $charset, tree => $_, xml => $xml)->render
-  } @$tree[($tree->[0] eq 'root' ? 1 : 4) .. $#$tree];
+  my $xml  = $self->xml;
+  return join '', map { _render($_, $xml) } _nodes($self->tree);
 }
 
-sub find {
-  my ($self, $selector) = @_;
+sub find { $_[0]->_collect(@{$_[0]->_css->select($_[1])}) }
 
-  my $charset = $self->charset;
-  my $xml     = $self->xml;
-  return Mojo::Collection->new(
-    map { $self->new->charset($charset)->tree($_)->xml($xml) }
-      @{Mojo::DOM::CSS->new(tree => $self->tree)->select($selector)});
-}
+sub match { $_[0]->_css->match($_[1]) ? $_[0] : undef }
 
 sub namespace {
   my $self = shift;
 
-  # Extract namespace prefix and search parents
   return '' if (my $current = $self->tree)->[0] eq 'root';
+
+  # Extract namespace prefix and search parents
   my $ns = $current->[1] =~ /^(.*?):/ ? "xmlns:$1" : undef;
-  while ($current) {
-    last if $current->[0] eq 'root';
+  while ($current->[0] ne 'root') {
 
     # Namespace for prefix
     my $attrs = $current->[2];
@@ -128,228 +104,244 @@ sub namespace {
     # Namespace attribute
     elsif (defined $attrs->{xmlns}) { return $attrs->{xmlns} }
 
-    # Parent
     $current = $current->[3];
   }
 
   return '';
 }
 
-sub next { shift->_sibling(1) }
+sub next { shift->_siblings->[1][0] }
 
 sub parent {
   my $self = shift;
   return undef if (my $tree = $self->tree)->[0] eq 'root';
-  return $self->new->charset($self->charset)->tree($tree->[3])
-    ->xml($self->xml);
+  return $self->new->tree($tree->[3])->xml($self->xml);
 }
 
-sub parse {
-  my $self = shift;
-  $self->[0]->parse(@_);
-  return $self;
-}
+sub parse { shift->_delegate(parse => shift) }
 
 sub prepend { shift->_add(0, @_) }
 
 sub prepend_content {
   my ($self, $new) = @_;
   my $tree = $self->tree;
-  splice @$tree, $tree->[0] eq 'root' ? 1 : 4, 0,
-    @{_parent($self->_parse("$new"), $tree)};
+  splice @$tree, _offset($tree), 0, _link($self->_parse("$new"), $tree);
   return $self;
 }
 
-sub previous { shift->_sibling(0) }
+sub previous { shift->_siblings->[0][-1] }
 
 sub remove { shift->replace('') }
 
 sub replace {
   my ($self, $new) = @_;
-
-  # Parse
   my $tree = $self->tree;
-  if   ($tree->[0] eq 'root') { return $self->xml(undef)->parse($new) }
-  else                        { $new = $self->_parse("$new") }
-
-  # Find and replace
-  my $parent = $tree->[3];
-  my $i = $parent->[0] eq 'root' ? 1 : 4;
-  for my $e (@$parent[$i .. $#$parent]) {
-    last if $e == $tree;
-    $i++;
-  }
-  splice @$parent, $i, 1, @{_parent($new, $parent)};
-
-  return $self;
+  return $self->xml(undef)->parse($new) if $tree->[0] eq 'root';
+  return $self->_replace($tree, $self->_parse("$new"));
 }
 
 sub replace_content {
   my ($self, $new) = @_;
   my $tree = $self->tree;
-  splice @$tree, $tree->[0] eq 'root' ? 1 : 4, $#$tree,
-    @{_parent($self->_parse("$new"), $tree)};
+  splice @$tree, _offset($tree), $#$tree, _link($self->_parse("$new"), $tree);
   return $self;
 }
 
 sub root {
   my $self = shift;
-
-  my $root = $self->tree;
-  while ($root->[0] eq 'tag') {
-    last unless my $parent = $root->[3];
-    $root = $parent;
-  }
-
-  return $self->new->charset($self->charset)->tree($root)->xml($self->xml);
+  return $self unless my $tree = _ancestors($self->tree, 1);
+  return $self->new->tree($tree)->xml($self->xml);
 }
 
-sub text {
-  my ($self, $trim) = @_;
+sub siblings { _select(Mojo::Collection->new(@{_siblings($_[0], 1)}), $_[1]) }
+
+sub strip {
+  my $self = shift;
   my $tree = $self->tree;
-  return _text(_elements($tree), 0, _trim($tree, $trim));
+  return $self if $tree->[0] eq 'root';
+  return $self->_replace($tree, ['root', _nodes($tree)]);
 }
 
+sub tap { shift->Mojo::Base::tap(@_) }
+
+sub text { shift->_content(0, @_) }
+
 sub text_after {
   my ($self, $trim) = @_;
 
-  # Find following text elements
   return '' if (my $tree = $self->tree)->[0] eq 'root';
-  my (@elements, $started);
-  for my $e (@{_elements($tree->[3])}) {
-    ++$started and next if $e eq $tree;
+
+  my (@nodes, $started);
+  for my $n (_nodes($tree->[3])) {
+    ++$started and next if $n eq $tree;
     next unless $started;
-    last if $e->[0] eq 'tag';
-    push @elements, $e;
+    last if $n->[0] eq 'tag';
+    push @nodes, $n;
   }
 
-  return _text(\@elements, 0, _trim($tree->[3], $trim));
+  return _text(\@nodes, 0, _trim($tree->[3], $trim));
 }
 
 sub text_before {
   my ($self, $trim) = @_;
 
-  # Find preceding text elements
   return '' if (my $tree = $self->tree)->[0] eq 'root';
-  my @elements;
-  for my $e (@{_elements($tree->[3])}) {
-    last if $e eq $tree;
-    push @elements, $e;
-    @elements = () if $e->[0] eq 'tag';
+
+  my @nodes;
+  for my $n (_nodes($tree->[3])) {
+    last if $n eq $tree;
+    push @nodes, $n;
+    @nodes = () if $n->[0] eq 'tag';
   }
 
-  return _text(\@elements, 0, _trim($tree->[3], $trim));
+  return _text(\@nodes, 0, _trim($tree->[3], $trim));
 }
 
 sub to_xml { shift->[0]->render }
 
-sub tree { shift->_html(tree => @_) }
+sub tree { shift->_delegate(tree => @_) }
 
 sub type {
   my ($self, $type) = @_;
-
-  # Get
   return '' if (my $tree = $self->tree)->[0] eq 'root';
   return $tree->[1] unless $type;
-
-  # Set
   $tree->[1] = $type;
-
   return $self;
 }
 
-sub xml { shift->_html(xml => @_) }
+sub xml { shift->_delegate(xml => @_) }
 
 sub _add {
   my ($self, $offset, $new) = @_;
 
-  # Not a tag
   return $self if (my $tree = $self->tree)->[0] eq 'root';
 
-  # Find parent
   my $parent = $tree->[3];
-  my $i = $parent->[0] eq 'root' ? 1 : 4;
-  for my $e (@$parent[$i .. $#$parent]) {
-    last if $e == $tree;
-    $i++;
-  }
-
-  # Add children
-  splice @$parent, $i + $offset, 0, @{_parent($self->_parse("$new"), $parent)};
+  splice @$parent, _parent($parent, $tree) + $offset, 0,
+    _link($self->_parse("$new"), $parent);
 
   return $self;
 }
 
-sub _elements {
-  return [] unless my $e = shift;
-  return [@$e[($e->[0] eq 'root' ? 1 : 4) .. $#$e]];
+sub _ancestors {
+  my ($tree, $root) = @_;
+  my @ancestors;
+  push @ancestors, $tree while ($tree->[0] eq 'tag') && ($tree = $tree->[3]);
+  return $root ? $ancestors[-1] : @ancestors[0 .. $#ancestors - 1];
 }
 
-sub _html {
+sub _collect {
+  my $self = shift;
+  my $xml  = $self->xml;
+  return Mojo::Collection->new(@_)
+    ->map(sub { $self->new->tree($_)->xml($xml) });
+}
+
+sub _content {
+  my $tree = shift->tree;
+  return _text([_nodes($tree)], shift, _trim($tree, @_));
+}
+
+sub _css { Mojo::DOM::CSS->new(tree => shift->tree) }
+
+sub _delegate {
   my ($self, $method) = (shift, shift);
   return $self->[0]->$method unless @_;
   $self->[0]->$method(@_);
   return $self;
 }
 
-sub _parent {
+sub _link {
   my ($children, $parent) = @_;
 
   # Link parent to children
   my @new;
-  for my $e (@$children[1 .. $#$children]) {
-    if ($e->[0] eq 'tag') {
-      $e->[3] = $parent;
-      weaken $e->[3];
-    }
-    push @new, $e;
+  for my $n (@$children[1 .. $#$children]) {
+    push @new, $n;
+    next unless $n->[0] eq 'tag';
+    $n->[3] = $parent;
+    weaken $n->[3];
   }
 
-  return \@new;
+  return @new;
 }
 
-sub _parse {
-  my $self = shift;
-  Mojo::DOM::HTML->new(charset => $self->charset, xml => $self->xml)
-    ->parse(shift)->tree;
+sub _nodes {
+  return unless my $n = shift;
+  return @$n[_offset($n) .. $#$n];
 }
 
-sub _sibling {
-  my ($self, $next) = @_;
+sub _offset { $_[0][0] eq 'root' ? 1 : 4 }
 
-  # Make sure we have a parent
-  return undef unless my $parent = $self->parent;
+sub _parent {
+  my ($parent, $child) = @_;
 
-  # Find previous or next sibling
-  my ($previous, $current);
+  # Find parent offset for child
+  my $i = _offset($parent);
+  for my $n (@$parent[$i .. $#$parent]) {
+    last if $n == $child;
+    $i++;
+  }
+
+  return $i;
+}
+
+sub _parse { Mojo::DOM::HTML->new(xml => shift->xml)->parse(shift)->tree }
+
+sub _render { Mojo::DOM::HTML->new(tree => shift, xml => shift)->render }
+
+sub _replace {
+  my ($self, $tree, $new) = @_;
+  my $parent = $tree->[3];
+  splice @$parent, _parent($parent, $tree), 1, _link($new, $parent);
+  return $self->parent;
+}
+
+sub _select {
+  my ($self, $selector) = @_;
+  return defined $selector ? $self->grep(sub { $_->match($selector) }) : $self;
+}
+
+sub _siblings {
+  my ($self, $merge) = @_;
+
+  return $merge ? [] : [[], []] unless my $parent = $self->parent;
+
+  my $tree = $self->tree;
+  my (@before, @after, $match);
   for my $child ($parent->children->each) {
-    ++$current and next if $child->tree eq $self->tree;
-    return $next ? $child : $previous if $current;
-    $previous = $child;
+    ++$match and next if $child->tree eq $tree;
+    $match ? push @after, $child : push @before, $child;
   }
 
-  # No siblings
-  return undef;
+  return $merge ? [@before, @after] : [\@before, \@after];
 }
 
 sub _text {
-  my ($elements, $recurse, $trim) = @_;
+  my ($nodes, $recurse, $trim) = @_;
+
+  # Merge successive text nodes
+  my $i = 0;
+  while (my $next = $nodes->[$i + 1]) {
+    ++$i and next unless $nodes->[$i][0] eq 'text' && $next->[0] eq 'text';
+    splice @$nodes, $i, 2, ['text', $nodes->[$i][1] . $next->[1]];
+  }
 
   my $text = '';
-  for my $e (@$elements) {
-    my $type = $e->[0];
+  for my $n (@$nodes) {
+    my $type = $n->[0];
 
     # Nested tag
     my $content = '';
     if ($type eq 'tag' && $recurse) {
-      $content = _text(_elements($e), 1, _trim($e, $trim));
+      $content = _text([_nodes($n)], 1, _trim($n, $trim));
     }
 
     # Text
-    elsif ($type eq 'text') { $content = $trim ? squish($e->[1]) : $e->[1] }
+    elsif ($type eq 'text') { $content = $trim ? squish($n->[1]) : $n->[1] }
 
     # CDATA or raw text
-    elsif ($type eq 'cdata' || $type eq 'raw') { $content = $e->[1] }
+    elsif ($type eq 'cdata' || $type eq 'raw') { $content = $n->[1] }
 
     # Add leading whitespace if punctuation allows it
     $content = " $content" if $text =~ /\S\z/ && $content =~ /^[^.!?,;:\s]+/;
@@ -378,6 +370,8 @@ sub _trim {
 
 1;
 
+=encoding utf8
+
 =head1 NAME
 
 Mojo::DOM - Minimalistic HTML/XML DOM parser with CSS selectors
@@ -391,7 +385,8 @@ Mojo::DOM - Minimalistic HTML/XML DOM parser with CSS selectors
 
   # Find
   say $dom->at('#b')->text;
-  say $dom->find('p')->pluck('text');
+  say $dom->find('p')->text;
+  say $dom->find('[id]')->attr('id');
 
   # Walk
   say $dom->div->p->[0]->text;
@@ -407,9 +402,10 @@ Mojo::DOM - Minimalistic HTML/XML DOM parser with CSS selectors
 
   # Modify
   $dom->div->p->[1]->append('<p id="c">C</p>');
+  $dom->find(':not(p)')->strip;
 
   # Render
-  say $dom;
+  say "$dom";
 
 =head1 DESCRIPTION
 
@@ -420,7 +416,7 @@ use it for validation.
 =head1 CASE SENSITIVITY
 
 L<Mojo::DOM> defaults to HTML semantics, that means all tags and attributes
-are lowercased and selectors need to be lower case as well.
+are lowercased and selectors need to be lowercase as well.
 
   my $dom = Mojo::DOM->new('<P ID="greeting">Hi!</P>');
   say $dom->at('p')->text;
@@ -433,7 +429,7 @@ into XML mode and everything becomes case sensitive.
   say $dom->at('P')->text;
   say $dom->P->{ID};
 
-XML detection can also be disabled with the C<xml> method.
+XML detection can also be disabled with the L</"xml"> method.
 
   # Force XML semantics
   $dom->xml(1);
@@ -443,16 +439,15 @@ XML detection can also be disabled with the C<xml> method.
 
 =head1 METHODS
 
-L<Mojo::DOM> inherits all methods from L<Mojo::Base> and implements the
-following new ones.
+L<Mojo::DOM> implements the following methods.
 
 =head2 new
 
   my $dom = Mojo::DOM->new;
   my $dom = Mojo::DOM->new('<foo bar="baz">test</foo>');
 
-Construct a new array-based L<Mojo::DOM> object and C<parse> HTML/XML document
-if necessary.
+Construct a new array-based L<Mojo::DOM> object and L</"parse"> HTML/XML
+fragment if necessary.
 
 =head2 all_text
 
@@ -468,11 +463,23 @@ enabled by default.
   # "foo\nbarbaz\n"
   $dom->parse("<div>foo\n<p>bar</p>baz\n</div>")->div->all_text(0);
 
+=head2 ancestors
+
+  my $collection = $dom->ancestors;
+  my $collection = $dom->ancestors('div');
+
+Find all ancestors of this element matching the CSS selector and return a
+L<Mojo::Collection> object containing these elements as L<Mojo::DOM> objects.
+All selectors from L<Mojo::DOM::CSS> are supported.
+
+  # List types of ancestor elements
+  say $dom->ancestors->type;
+
 =head2 append
 
   $dom = $dom->append('<p>Hi!</p>');
 
-Append to element.
+Append HTML/XML fragment to element.
 
   # "<div><h1>A</h1><h2>B</h2></div>"
   $dom->parse('<div><h1>A</h1></div>')->at('h1')->append('<h2>B</h2>')->root;
@@ -481,7 +488,7 @@ Append to element.
 
   $dom = $dom->append_content('<p>Hi!</p>');
 
-Append to element content.
+Append HTML/XML fragment to element content.
 
   # "<div><h1>AB</h1></div>"
   $dom->parse('<div><h1>A</h1></div>')->at('h1')->append_content('B')->root;
@@ -497,29 +504,26 @@ L<Mojo::DOM::CSS> are supported.
   # Find first element with "svg" namespace definition
   my $namespace = $dom->at('[xmlns\:svg]')->{'xmlns:svg'};
 
-=head2 attrs
+=head2 attr
 
-  my $attrs = $dom->attrs;
-  my $foo   = $dom->attrs('foo');
-  $dom      = $dom->attrs({foo => 'bar'});
-  $dom      = $dom->attrs(foo => 'bar');
+  my $attrs = $dom->attr;
+  my $foo   = $dom->attr('foo');
+  $dom      = $dom->attr({foo => 'bar'});
+  $dom      = $dom->attr(foo => 'bar');
 
 Element attributes.
 
-=head2 charset
-
-  my $charset = $dom->charset;
-  $dom        = $dom->charset('UTF-8');
-
-Charset used for decoding and encoding HTML/XML.
+  # List id attributes
+  say $dom->find('*')->attr('id')->compact;
 
 =head2 children
 
   my $collection = $dom->children;
   my $collection = $dom->children('div');
 
-Return a L<Mojo::Collection> object containing the children of this element as
-L<Mojo::DOM> objects, similar to C<find>.
+Find all children of this element matching the CSS selector and return a
+L<Mojo::Collection> object containing these elements as L<Mojo::DOM> objects.
+All selectors from L<Mojo::DOM::CSS> are supported.
 
   # Show type of random child element
   say $dom->children->shuffle->first->type;
@@ -528,8 +532,7 @@ L<Mojo::DOM> objects, similar to C<find>.
 
   my $xml = $dom->content_xml;
 
-Render content of this element to XML. Note that the XML will be encoded if a
-C<charset> has been defined.
+Render content of this element to XML.
 
   # "<b>test</b>"
   $dom->parse('<div><b>test</b></div>')->div->content_xml;
@@ -546,7 +549,16 @@ L<Mojo::DOM::CSS> are supported.
   my $id = $dom->find('div')->[23]{id};
 
   # Extract information from multiple elements
-  my @headers = $dom->find('h1, h2, h3')->pluck('text')->each;
+  my @headers = $dom->find('h1, h2, h3')->text->each;
+  my @links   = $dom->find('a[href]')->attr('href')->each;
+
+=head2 match
+
+  my $result = $dom->match('html title');
+
+Match the CSS selector against this element and return it as a L<Mojo::DOM>
+object or return C<undef> if it didn't match. All selectors from
+L<Mojo::DOM::CSS> are supported.
 
 =head2 namespace
 
@@ -581,16 +593,16 @@ has no parent.
 
   $dom = $dom->parse('<foo bar="baz">test</foo>');
 
-Parse HTML/XML document with L<Mojo::DOM::HTML>.
+Parse HTML/XML fragment with L<Mojo::DOM::HTML>.
 
-  # Parse UTF-8 encoded XML
-  my $dom = Mojo::DOM->new->charset('UTF-8')->xml(1)->parse($xml);
+  # Parse XML
+  my $dom = Mojo::DOM->new->xml(1)->parse($xml);
 
 =head2 prepend
 
   $dom = $dom->prepend('<p>Hi!</p>');
 
-Prepend to element.
+Prepend HTML/XML fragment to element.
 
   # "<div><h1>A</h1><h2>B</h2></div>"
   $dom->parse('<div><h2>B</h2></div>')->at('h2')->prepend('<h1>A</h1>')->root;
@@ -599,7 +611,7 @@ Prepend to element.
 
   $dom = $dom->prepend_content('<p>Hi!</p>');
 
-Prepend to element content.
+Prepend HTML/XML fragment to element content.
 
   # "<div><h2>AB</h2></div>"
   $dom->parse('<div><h2>B</h2></div>')->at('h2')->prepend_content('A')->root;
@@ -616,30 +628,31 @@ there are no more siblings.
 
 =head2 remove
 
-  my $old = $dom->remove;
+  my $parent = $dom->remove;
 
-Remove element and return it as a L<Mojo::DOM> object.
+Remove element and return L<Mojo::DOM> object for parent of element.
 
   # "<div></div>"
-  $dom->parse('<div><h1>A</h1></div>')->at('h1')->remove->root;
+  $dom->parse('<div><h1>A</h1></div>')->at('h1')->remove;
 
 =head2 replace
 
-  my $old = $dom->replace('<div>test</div>');
+  my $parent = $dom->replace('<div>test</div>');
 
-Replace element and return the replaced element as a L<Mojo::DOM> object.
+Replace element with HTML/XML fragment and return L<Mojo::DOM> object for
+parent of element.
 
   # "<div><h2>B</h2></div>"
-  $dom->parse('<div><h1>A</h1></div>')->at('h1')->replace('<h2>B</h2>')->root;
+  $dom->parse('<div><h1>A</h1></div>')->at('h1')->replace('<h2>B</h2>');
 
   # "<div></div>"
-  $dom->parse('<div><h1>A</h1></div>')->at('h1')->replace('')->root;
+  $dom->parse('<div><h1>A</h1></div>')->at('h1')->replace('');
 
 =head2 replace_content
 
-  $dom = $dom->replace_content('test');
+  $dom = $dom->replace_content('<p>test</p>');
 
-Replace element content.
+Replace element content with HTML/XML fragment.
 
   # "<div><h1>B</h1></div>"
   $dom->parse('<div><h1>A</h1></div>')->at('h1')->replace_content('B')->root;
@@ -653,6 +666,34 @@ Replace element content.
 
 Return L<Mojo::DOM> object for root node.
 
+=head2 siblings
+
+  my $collection = $dom->siblings;
+  my $collection = $dom->siblings('div');
+
+Find all siblings of this element matching the CSS selector and return a
+L<Mojo::Collection> object containing these elements as L<Mojo::DOM> objects.
+All selectors from L<Mojo::DOM::CSS> are supported.
+
+  # List types of sibling elements
+  say $dom->siblings->type;
+
+=head2 strip
+
+  my $parent = $dom->strip;
+
+Remove element while preserving its content and return L<Mojo::DOM> object for
+parent of element.
+
+  # "<div>A</div>"
+  $dom->parse('<div><h1>A</h1></div>')->at('h1')->strip;
+
+=head2 tap
+
+  $dom = $dom->tap(sub {...});
+
+Alias for L<Mojo::Base/"tap">.
+
 =head2 text
 
   my $trimmed   = $dom->text;
@@ -700,8 +741,7 @@ is enabled by default.
   my $xml = $dom->to_xml;
   my $xml = "$dom";
 
-Render this element and its content to XML. Note that the XML will be encoded
-if a C<charset> has been defined.
+Render this element and its content to XML.
 
   # "<b>test</b>"
   $dom->parse('<div><b>test</b></div>')->div->b->to_xml;
@@ -709,7 +749,7 @@ if a C<charset> has been defined.
 =head2 tree
 
   my $tree = $dom->tree;
-  $dom     = $dom->tree(['root', [qw(text lalala)]]);
+  $dom     = $dom->tree(['root', ['text', 'foo']]);
 
 Document Object Model. Note that this structure should only be used very
 carefully since it is very dynamic.
@@ -722,12 +762,12 @@ carefully since it is very dynamic.
 Element type.
 
   # List types of child elements
-  say $dom->children->pluck('type');
+  say $dom->children->type;
 
 =head2 xml
 
-  my $xml = $dom->xml;
-  $dom    = $dom->xml(1);
+  my $bool = $dom->xml;
+  $dom     = $dom->xml($bool);
 
 Disable HTML semantics in parser and activate case sensitivity, defaults to
 auto detection based on processing instructions.
@@ -740,7 +780,7 @@ L<Mojo::Collection> object, depending on number of children.
 
   say $dom->p->text;
   say $dom->div->[23]->text;
-  say $dom->div->pluck('text');
+  say $dom->div->text;
 
 =head1 ELEMENT ATTRIBUTES
 
