@@ -1,24 +1,23 @@
 package Mojo::URL;
 use Mojo::Base -base;
-use overload
-  'bool'   => sub {1},
-  '""'     => sub { shift->to_string },
-  fallback => 1;
+use overload bool => sub {1}, '""' => sub { shift->to_string }, fallback => 1;
 
 use Mojo::Parameters;
 use Mojo::Path;
-use Mojo::Util qw(punycode_decode punycode_encode url_escape url_unescape);
+use Mojo::Util
+  qw(deprecated punycode_decode punycode_encode url_escape url_unescape);
 
 has base => sub { Mojo::URL->new };
-has [qw(data fragment host port scheme userinfo)];
+has [qw(fragment host port scheme userinfo)];
 
 sub new { shift->SUPER::new->parse(@_) }
 
 sub authority {
-  my ($self, $authority) = @_;
+  my $self = shift;
 
   # New authority
-  if (defined $authority) {
+  if (@_) {
+    return $self unless defined(my $authority = shift);
 
     # Userinfo
     $authority =~ s/^([^\@]+)\@// and $self->userinfo(url_unescape $1);
@@ -32,10 +31,11 @@ sub authority {
   }
 
   # Build authority
-  my $userinfo = $self->userinfo;
-  $authority .= url_escape($userinfo, '^A-Za-z0-9\-._~!$&\'()*+,;=:') . '@'
-    if $userinfo;
-  $authority .= $self->ihost // '';
+  return undef unless defined(my $authority = $self->ihost);
+  if (my $userinfo = $self->userinfo) {
+    $userinfo = url_escape $userinfo, '^A-Za-z0-9\-._~!$&\'()*+,;=:';
+    $authority = $userinfo . '@' . $authority;
+  }
   if (my $port = $self->port) { $authority .= ":$port" }
 
   return $authority;
@@ -45,7 +45,7 @@ sub clone {
   my $self = shift;
 
   my $clone = $self->new;
-  $clone->$_($self->$_) for qw(scheme data userinfo host port fragment);
+  $clone->$_($self->$_) for qw(scheme userinfo host port fragment);
   $clone->path($self->path->clone);
   $clone->query($self->query->clone);
   $clone->base($self->base->clone) if $self->{base};
@@ -62,7 +62,7 @@ sub ihost {
     if @_;
 
   # Check if host needs to be encoded
-  return undef unless my $host = $self->host;
+  return undef unless defined(my $host = $self->host);
   return lc $host unless $host =~ /[^\x00-\x7f]/;
 
   # Encode
@@ -77,31 +77,20 @@ sub parse {
   my ($self, $url) = @_;
   return $self unless $url;
 
-  # Official regex
-  $url =~ m!(?:([^:/?#]+):)?(?://([^/?#]*))?([^?#]*)(?:\?([^#]*))?(?:#(.*))?!;
-
-  # Supported scheme
-  my $proto = $self->scheme($1)->protocol;
-  if (!$proto || grep { $proto eq $_ } qw(http https ws wss)) {
-    $self->authority($2);
-    $self->path->parse($3);
-    $self->query($4)->fragment($5);
-  }
-
-  # Preserve scheme data
-  else { $self->data(substr($url, length($proto) + 1)) }
-
-  return $self;
+  # Official regex from RFC 3986
+  $url =~ m!^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?!;
+  return $self->scheme($2)->authority($4)->path($5)->query($7)->fragment($9);
 }
 
 sub path {
-  my ($self, $path) = @_;
+  my $self = shift;
 
   # Old path
   $self->{path} ||= Mojo::Path->new;
-  return $self->{path} unless $path;
+  return $self->{path} unless @_;
 
   # New path
+  my $path = shift;
   $self->{path} = ref $path ? $path : $self->{path}->merge($path);
 
   return $self;
@@ -171,7 +160,9 @@ sub to_abs {
   return $abs;
 }
 
+# DEPRECATED in Top Hat!
 sub to_rel {
+  deprecated 'Mojo::URL::to_rel is DEPRECATED';
   my $self = shift;
 
   my $rel = $self->clone;
@@ -188,8 +179,7 @@ sub to_rel {
   my @base_parts = @{$base_path->parts};
   pop @base_parts unless $base_path->trailing_slash;
   while (@parts && @base_parts && $parts[0] eq $base_parts[0]) {
-    shift @parts;
-    shift @base_parts;
+    shift @$_ for \@parts, \@base_parts;
   }
   my $path = $rel->path(Mojo::Path->new)->path;
   $path->leading_slash(1) if $rel->authority;
@@ -202,36 +192,24 @@ sub to_rel {
 sub to_string {
   my $self = shift;
 
-  # Scheme data
-  my $data = $self->data;
-  return join ':', $self->scheme, $data if defined $data;
-
-  # Protocol
+  # Scheme
   my $url = '';
-  if (my $proto = $self->protocol) { $url .= "$proto://" }
+  if (my $proto = $self->protocol) { $url .= "$proto:" }
 
   # Authority
   my $authority = $self->authority;
-  $url .= $url ? $authority : $authority ? "//$authority" : '';
+  $url .= "//$authority" if defined $authority;
 
-  # Relative path
-  my $path = $self->path;
-  if (!$url) { $url .= "$path" }
-
-  # Absolute path
-  elsif ($path->leading_slash) { $url .= "$path" }
-  else                         { $url .= @{$path->parts} ? "/$path" : '' }
+  # Path
+  my $path = $self->path->to_string;
+  $url .= !$authority || $path eq '' || $path =~ m!^/! ? $path : "/$path";
 
   # Query
-  my $query = join '', $self->query;
-  $url .= "?$query" if length $query;
+  if (length(my $query = $self->query->to_string)) { $url .= "?$query" }
 
   # Fragment
-  my $fragment = $self->fragment;
-  $url .= '#' . url_escape $fragment, '^A-Za-z0-9\-._~!$&\'()*+,;=%:@/?'
-    if $fragment;
-
-  return $url;
+  return $url unless defined(my $fragment = $self->fragment);
+  return $url . '#' . url_escape $fragment, '^A-Za-z0-9\-._~!$&\'()*+,;=%:@/?';
 }
 
 1;
@@ -264,7 +242,6 @@ Mojo::URL - Uniform Resource Locator
   $url->host('example.com');
   $url->port(3000);
   $url->path('/foo/bar');
-  $url->path('baz');
   $url->query->param(foo => 'bar');
   $url->fragment(23);
   say "$url";
@@ -283,17 +260,7 @@ L<Mojo::URL> implements the following attributes.
   my $base = $url->base;
   $url     = $url->base(Mojo::URL->new);
 
-Base of this URL.
-
-=head2 data
-
-  my $data = $url->data;
-  $url     = $url->data('foo')
-
-Data preserved for unknown schemes.
-
-  # "sri@example.com"
-  Mojo::URL->new('mailto:sri@example.com')->data;
+Base of this URL, defaults to a L<Mojo::URL> object.
 
 =head2 fragment
 
@@ -340,7 +307,7 @@ following new ones.
   my $url = Mojo::URL->new;
   my $url = Mojo::URL->new('http://127.0.0.1:3000/foo?f=b&baz=2#foo');
 
-Construct a new L<Mojo::URL> object and C<parse> URL if necessary.
+Construct a new L<Mojo::URL> object and L</"parse"> URL if necessary.
 
 =head2 authority
 
@@ -367,7 +334,7 @@ Host part of this URL in punycode format.
 
 =head2 is_abs
 
-  my $success = $url->is_abs;
+  my $bool = $url->is_abs;
 
 Check if URL is absolute.
 
@@ -375,8 +342,7 @@ Check if URL is absolute.
 
   $url = $url->parse('http://127.0.0.1:3000/foo/bar?fo=o&baz=23#foo');
 
-Parse relative or absolute URL for the C<http>, C<https>, C<ws> as well as
-C<wss> schemes and preserve scheme data for all unknown ones.
+Parse relative or absolute URL.
 
   # "/test/123"
   $url->parse('/test/123?foo=bar')->path;
@@ -384,8 +350,8 @@ C<wss> schemes and preserve scheme data for all unknown ones.
   # "example.com"
   $url->parse('http://example.com/test/123?foo=bar')->host;
 
-  # "mailto:sri@example.com"
-  $url->parse('mailto:sri@example.com')->to_string;
+  # "sri@example.com"
+  $url->parse('mailto:sri@example.com')->path;
 
 =head2 path
 
@@ -397,23 +363,23 @@ C<wss> schemes and preserve scheme data for all unknown ones.
 Path part of this URL, relative paths will be merged with the existing path,
 defaults to a L<Mojo::Path> object.
 
-  # "http://mojolicio.us/DOM/HTML"
-  Mojo::URL->new('http://mojolicio.us/perldoc/Mojo')->path('/DOM/HTML');
+  # "http://example.com/DOM/HTML"
+  Mojo::URL->new('http://example.com/perldoc/Mojo')->path('/DOM/HTML');
 
-  # "http://mojolicio.us/perldoc/DOM/HTML"
-  Mojo::URL->new('http://mojolicio.us/perldoc/Mojo')->path('DOM/HTML');
+  # "http://example.com/perldoc/DOM/HTML"
+  Mojo::URL->new('http://example.com/perldoc/Mojo')->path('DOM/HTML');
 
-  # "http://mojolicio.us/perldoc/Mojo/DOM/HTML"
-  Mojo::URL->new('http://mojolicio.us/perldoc/Mojo/')->path('DOM/HTML');
+  # "http://example.com/perldoc/Mojo/DOM/HTML"
+  Mojo::URL->new('http://example.com/perldoc/Mojo/')->path('DOM/HTML');
 
 =head2 protocol
 
   my $proto = $url->protocol;
 
-Normalized version of C<scheme>.
+Normalized version of L</"scheme">.
 
   # "http"
-  Mojo::URL->new('HtTp://mojolicio.us')->protocol;
+  Mojo::URL->new('HtTp://example.com')->protocol;
 
 =head2 query
 
@@ -427,41 +393,47 @@ Query part of this URL, pairs in an array will be merged and pairs in a hash
 appended, defaults to a L<Mojo::Parameters> object.
 
   # "2"
-  Mojo::URL->new('http://mojolicio.us?a=1&b=2')->query->param('b');
+  Mojo::URL->new('http://example.com?a=1&b=2')->query->param('b');
 
-  # "http://mojolicio.us?a=2&c=3"
-  Mojo::URL->new('http://mojolicio.us?a=1&b=2')->query(a => 2, c => 3);
+  # "http://example.com?a=2&c=3"
+  Mojo::URL->new('http://example.com?a=1&b=2')->query(a => 2, c => 3);
 
-  # "http://mojolicio.us?a=2&a=3"
-  Mojo::URL->new('http://mojolicio.us?a=1&b=2')->query(a => [2, 3]);
+  # "http://example.com?a=2&a=3"
+  Mojo::URL->new('http://example.com?a=1&b=2')->query(a => [2, 3]);
 
-  # "http://mojolicio.us?a=2&b=2&c=3"
-  Mojo::URL->new('http://mojolicio.us?a=1&b=2')->query([a => 2, c => 3]);
+  # "http://example.com?a=2&b=2&c=3"
+  Mojo::URL->new('http://example.com?a=1&b=2')->query([a => 2, c => 3]);
 
-  # "http://mojolicio.us?b=2"
-  Mojo::URL->new('http://mojolicio.us?a=1&b=2')->query([a => undef]);
+  # "http://example.com?b=2"
+  Mojo::URL->new('http://example.com?a=1&b=2')->query([a => undef]);
 
-  # "http://mojolicio.us?a=1&b=2&a=2&c=3"
-  Mojo::URL->new('http://mojolicio.us?a=1&b=2')->query({a => 2, c => 3});
+  # "http://example.com?a=1&b=2&a=2&c=3"
+  Mojo::URL->new('http://example.com?a=1&b=2')->query({a => 2, c => 3});
 
 =head2 to_abs
 
   my $abs = $url->to_abs;
   my $abs = $url->to_abs(Mojo::URL->new('http://example.com/foo'));
 
-Clone relative URL and turn it into an absolute one.
+Clone relative URL and turn it into an absolute one using L</"base"> or
+provided base URL.
 
-=head2 to_rel
+  # "http://example.com/foo/baz.xml?test=123"
+  Mojo::URL->new('baz.xml?test=123')
+    ->to_abs(Mojo::URL->new('http://example.com/foo/bar.html'));
 
-  my $rel = $url->to_rel;
-  my $rel = $url->to_rel(Mojo::URL->new('http://example.com/foo'));
+  # "http://example.com/baz.xml?test=123"
+  Mojo::URL->new('/baz.xml?test=123')
+    ->to_abs(Mojo::URL->new('http://example.com/foo/bar.html'));
 
-Clone absolute URL and turn it into a relative one.
+  # "http://example.com/foo/baz.xml?test=123"
+  Mojo::URL->new('//example.com/foo/baz.xml?test=123')
+    ->to_abs(Mojo::URL->new('http://example.com/foo/bar.html'));
 
 =head2 to_string
 
-  my $string = $url->to_string;
-  my $string = "$url";
+  my $str = $url->to_string;
+  my $str = "$url";
 
 Turn URL into a string.
 
