@@ -4,12 +4,12 @@ use Mojo::Base 'Mojo::EventEmitter';
 # "Fry: Since when is the Internet about robbing people of their privacy?
 #  Bender: August 6, 1991."
 use Carp 'croak';
-use List::Util 'first';
 use Mojo::IOLoop;
-use Mojo::Server::Daemon;
 use Mojo::URL;
 use Mojo::Util qw(deprecated monkey_patch);
 use Mojo::UserAgent::CookieJar;
+use Mojo::UserAgent::Proxy;
+use Mojo::UserAgent::Server;
 use Mojo::UserAgent::Transactor;
 use Scalar::Util 'weaken';
 
@@ -19,14 +19,15 @@ has ca              => sub { $ENV{MOJO_CA_FILE} };
 has cert            => sub { $ENV{MOJO_CERT_FILE} };
 has connect_timeout => sub { $ENV{MOJO_CONNECT_TIMEOUT} || 10 };
 has cookie_jar      => sub { Mojo::UserAgent::CookieJar->new };
-has [qw(http_proxy https_proxy local_address no_proxy)];
+has 'local_address';
 has inactivity_timeout => sub { $ENV{MOJO_INACTIVITY_TIMEOUT} // 20 };
 has ioloop             => sub { Mojo::IOLoop->new };
 has key                => sub { $ENV{MOJO_KEY_FILE} };
 has max_connections    => 5;
 has max_redirects => sub { $ENV{MOJO_MAX_REDIRECTS} || 0 };
-has name => 'Mojolicious (Perl)';
+has proxy => sub { Mojo::UserAgent::Proxy->new };
 has request_timeout => sub { $ENV{MOJO_REQUEST_TIMEOUT} // 0 };
+has server => sub { Mojo::UserAgent::Server->new(ioloop => shift->ioloop) };
 has transactor => sub { Mojo::UserAgent::Transactor->new };
 
 # Common HTTP methods
@@ -40,83 +41,89 @@ for my $name (qw(DELETE GET HEAD OPTIONS PATCH POST PUT)) {
 
 sub DESTROY { shift->_cleanup }
 
-sub app {
-  my ($self, $app) = @_;
-
-  # Singleton application
-  state $singleton;
-  return $singleton = $app ? $app : $singleton unless ref $self;
-
-  # Default to singleton application
-  return $self->{app} || $singleton unless $app;
-  $self->{app} = $app;
+# DEPRECATED in Top Hat!
+sub new {
+  my $self = shift->SUPER::new(@_);
+  for my $key (keys %$self) { $self->$key(delete $self->{$key}) }
   return $self;
 }
 
+# DEPRECATED in Top Hat!
+sub app {
+  deprecated 'Mojo::UserAgent::app is DEPRECATED in favor of'
+    . ' Mojo::UserAgent::Server::app';
+  shift->_delegate('server', 'app', @_);
+}
+
+# DEPRECATED in Top Hat!
 sub app_url {
-  my $self = shift;
-  $self->_server(@_);
-  return Mojo::URL->new("$self->{proto}://localhost:$self->{port}/");
-}
-
-# DEPRECATED in Rainbow!
-sub build_form_tx {
-  deprecated 'Mojo::UserAgent::build_form_tx is DEPRECATED in favor of '
-    . 'Mojo::UserAgent::build_tx';
-  shift->transactor->form(@_);
-}
-
-# DEPRECATED in Rainbow!
-sub build_json_tx {
-  deprecated 'Mojo::UserAgent::build_json_tx is DEPRECATED in favor of '
-    . 'Mojo::UserAgent::build_tx';
-  shift->transactor->json(@_);
+  deprecated 'Mojo::UserAgent::app_url is DEPRECATED in favor of'
+    . ' Mojo::UserAgent::Server::url';
+  shift->_delegate('server', 'url', @_);
 }
 
 sub build_tx           { shift->transactor->tx(@_) }
 sub build_websocket_tx { shift->transactor->websocket(@_) }
 
+# DEPRECATED in Top Hat!
 sub detect_proxy {
-  my $self = shift;
-  $self->http_proxy($ENV{HTTP_PROXY}   || $ENV{http_proxy});
-  $self->https_proxy($ENV{HTTPS_PROXY} || $ENV{https_proxy});
-  return $self->no_proxy([split /,/, $ENV{NO_PROXY} || $ENV{no_proxy} || '']);
+  deprecated 'Mojo::UserAgent::detect_proxy is DEPRECATED in favor of'
+    . ' Mojo::UserAgent::Proxy::detect';
+  shift->tap(sub { $_->proxy->detect });
 }
 
+# DEPRECATED in Top Hat!
+sub http_proxy {
+  deprecated 'Mojo::UserAgent::http_proxy is DEPRECATED in favor of'
+    . ' Mojo::UserAgent::Proxy::http';
+  shift->_delegate('proxy', 'http', @_);
+}
+
+# DEPRECATED in Top Hat!
+sub https_proxy {
+  deprecated 'Mojo::UserAgent::https_proxy is DEPRECATED in favor of'
+    . ' Mojo::UserAgent::Proxy::https';
+  shift->_delegate('proxy', 'https', @_);
+}
+
+# DEPRECATED in Top Hat!
+sub name {
+  deprecated 'Mojo::UserAgent::name is DEPRECATED in favor of'
+    . ' Mojo::UserAgent::Transactor::name';
+  shift->_delegate('transactor', 'name', @_);
+}
+
+# DEPRECATED in Top Hat!
+sub no_proxy {
+  deprecated 'Mojo::UserAgent::no_proxy is DEPRECATED in favor of'
+    . ' Mojo::UserAgent::Proxy::not';
+  shift->_delegate('proxy', 'not', @_);
+}
+
+# DEPRECATED in Top Hat!
 sub need_proxy {
-  my ($self, $host) = @_;
-  return !first { $host =~ /\Q$_\E$/ } @{$self->no_proxy || []};
-}
-
-# DEPRECATED in Rainbow!
-sub post_form {
-  deprecated 'Mojo::UserAgent::post_form is DEPRECATED in favor of '
-    . 'Mojo::UserAgent::post';
-  my $self = shift;
-  my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-  return $self->start($self->build_form_tx(@_), $cb);
-}
-
-# DEPRECATED in Rainbow!
-sub post_json {
-  deprecated 'Mojo::UserAgent::post_json is DEPRECATED in favor of '
-    . 'Mojo::UserAgent::post';
-  my $self = shift;
-  my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
-  return $self->start($self->build_json_tx(@_), $cb);
+  deprecated 'Mojo::UserAgent::need_proxy is DEPRECATED in favor of'
+    . ' Mojo::UserAgent::Proxy::is_needed';
+  shift->proxy->is_needed(@_);
 }
 
 sub start {
   my ($self, $tx, $cb) = @_;
 
+  # Fork safety
+  unless (($self->{pid} //= $$) eq $$) {
+    delete $self->_cleanup->{pid};
+    $self->server->restart;
+  }
+
   # Non-blocking
   if ($cb) {
     warn "-- Non-blocking request (@{[$tx->req->url->to_abs]})\n" if DEBUG;
     unless ($self->{nb}) {
       croak 'Blocking request in progress' if keys %{$self->{connections}};
       warn "-- Switching to non-blocking mode\n" if DEBUG;
-      $self->_cleanup;
-      $self->{nb}++;
+      $self->server->ioloop(Mojo::IOLoop->singleton);
+      $self->_cleanup->{nb}++;
     }
     return $self->_start($tx, $cb);
   }
@@ -126,8 +133,8 @@ sub start {
   if ($self->{nb}) {
     croak 'Non-blocking requests in progress' if keys %{$self->{connections}};
     warn "-- Switching to blocking mode\n" if DEBUG;
-    $self->_cleanup;
-    delete $self->{nb};
+    $self->server->ioloop($self->ioloop);
+    delete $self->_cleanup->{nb};
   }
   $self->_start($tx => sub { shift->ioloop->stop; $tx = shift });
   $self->ioloop->start;
@@ -141,50 +148,17 @@ sub websocket {
   $self->start($self->build_websocket_tx(@_), $cb);
 }
 
-sub _cache {
-  my ($self, $name, $id) = @_;
-
-  # Enqueue and enforce connection limit
-  my $old = $self->{cache} ||= [];
-  if ($id) {
-    my $max = $self->max_connections;
-    $self->_remove(shift(@$old)->[1]) while @$old > $max;
-    push @$old, [$name, $id] if $max;
-    return undef;
-  }
-
-  # Dequeue
-  my $found;
-  my $loop = $self->_loop;
-  my $new = $self->{cache} = [];
-  for my $cached (@$old) {
-
-    # Search for id/name and remove corrupted connections
-    if (!$found && ($cached->[1] eq $name || $cached->[0] eq $name)) {
-      my $stream = $loop->stream($cached->[1]);
-      if ($stream && !$stream->is_readable) { $found = $cached->[1] }
-      else                                  { $loop->remove($cached->[1]) }
-    }
-
-    # Requeue
-    else { push @$new, $cached }
-  }
-
-  return $found;
-}
-
 sub _cleanup {
   my $self = shift;
   return unless my $loop = $self->_loop;
 
   # Clean up active connections (by closing them)
-  $self->_handle($_ => 1) for keys %{$self->{connections} || {}};
+  $self->_handle($_, 1) for keys %{$self->{connections} || {}};
 
   # Clean up keep-alive connections
-  $loop->remove($_->[1]) for @{delete $self->{cache} || []};
+  $loop->remove($_->[1]) for @{delete $self->{queue} || []};
 
-  # Stop server
-  delete $self->{server};
+  return $self;
 }
 
 sub _connect {
@@ -198,7 +172,7 @@ sub _connect {
     local_address => $self->local_address,
     port          => $port,
     timeout       => $self->connect_timeout,
-    tls           => $proto eq 'https' ? 1 : 0,
+    tls           => $proto eq 'https',
     tls_ca        => $self->ca,
     tls_cert      => $self->cert,
     tls_key       => $self->key,
@@ -210,10 +184,11 @@ sub _connect {
       return $self->_error($id, $err) if $err;
 
       # Connection established
-      $stream->on(timeout => sub { $self->_error($id, 'Inactivity timeout') });
-      $stream->on(close => sub { $self->_handle($id => 1) });
-      $stream->on(error => sub { $self && $self->_error($id, pop, 1) });
-      $stream->on(read => sub { $self->_read($id => pop) });
+      $stream->on(
+        timeout => sub { $self->_error($id, 'Inactivity timeout', 1) });
+      $stream->on(close => sub { $self->_handle($id, 1) });
+      $stream->on(error => sub { $self && $self->_error($id, pop) });
+      $stream->on(read => sub { $self->_read($id, pop) });
       $cb->();
     }
   );
@@ -228,19 +203,19 @@ sub _connect_proxy {
     $new => sub {
       my ($self, $tx) = @_;
 
-      # CONNECT failed
-      unless (($tx->res->code // '') eq '200') {
+      # CONNECT failed (connection needs to be kept alive)
+      unless ($tx->keep_alive && $tx->res->is_status_class(200)) {
         $old->req->error('Proxy connection failed');
-        return $self->_finish($old, $cb);
+        return $self->$cb($old);
       }
 
       # Prevent proxy reassignment and start real transaction
       $old->req->proxy(0);
-      return $self->_start($old->connection($tx->connection), $cb)
+      my $id = $tx->connection;
+      return $self->_start($old->connection($id), $cb)
         unless $tx->req->url->protocol eq 'https';
 
       # TLS upgrade
-      return unless my $id = $tx->connection;
       my $loop   = $self->_loop;
       my $handle = $loop->stream($id)->steal_handle;
       my $c      = delete $self->{connections}{$id};
@@ -277,7 +252,7 @@ sub _connection {
   # Reuse connection
   my $id = $tx->connection;
   my ($proto, $host, $port) = $self->transactor->endpoint($tx);
-  $id ||= $self->_cache("$proto:$host:$port");
+  $id ||= $self->_dequeue("$proto:$host:$port", 1);
   if ($id && !ref $id) {
     warn "-- Reusing connection ($proto:$host:$port)\n" if DEBUG;
     $self->{connections}{$id} = {cb => $cb, tx => $tx};
@@ -300,40 +275,56 @@ sub _connection {
   return $id;
 }
 
-sub _error {
-  my ($self, $id, $err, $emit) = @_;
-  if (my $tx = $self->{connections}{$id}{tx}) { $tx->res->error($err) }
-  $self->emit(error => $err) if $emit;
-  $self->_handle($id => $err);
+# DEPRECATED in Top Hat!
+sub _delegate {
+  my ($self, $attr, $name) = (shift, shift, shift);
+  return $self->$attr->$name unless @_;
+  $self->$attr->$name(@_);
+  return $self;
 }
 
-sub _finish {
-  my ($self, $tx, $cb, $close) = @_;
+sub _dequeue {
+  my ($self, $name, $test) = @_;
 
-  # Remove code from parser errors
-  my $res = $tx->res;
-  if (my $err = $res->error) { $res->error($err) }
+  my $found;
+  my $loop = $self->_loop;
+  my $old  = $self->{queue} || [];
+  my $new  = $self->{queue} = [];
+  for my $queued (@$old) {
+    push @$new, $queued and next if $found || !grep { $_ eq $name } @$queued;
 
-  else {
-
-    # Premature connection close
-    if ($close && !$res->code) { $res->error('Premature connection close') }
-
-    # 400/500
-    elsif ($res->is_status_class(400) || $res->is_status_class(500)) {
-      $res->error($res->message, $res->code);
-    }
+    # Search for id/name and sort out corrupted connections if necessary
+    next unless my $stream = $loop->stream($queued->[1]);
+    $test && $stream->is_readable ? $stream->close : ($found = $queued->[1]);
   }
 
-  $self->$cb($tx);
+  return $found;
+}
+
+sub _enqueue {
+  my ($self, $name, $id) = @_;
+
+  # Enforce connection limit
+  my $queue = $self->{queue} ||= [];
+  my $max = $self->max_connections;
+  $self->_remove(shift(@$queue)->[1]) while @$queue > $max;
+  push @$queue, [$name, $id] if $max;
+}
+
+sub _error {
+  my ($self, $id, $err, $timeout) = @_;
+  if (my $tx = $self->{connections}{$id}{tx}) { $tx->res->error($err) }
+  elsif (!$timeout) { return $self->emit(error => $err) }
+  $self->_handle($id, 1);
 }
 
 sub _handle {
   my ($self, $id, $close) = @_;
 
   # Remove request timeout
+  return unless my $loop = $self->_loop;
   my $c = $self->{connections}{$id};
-  $self->_loop->remove($c->{timeout}) if $c->{timeout};
+  $loop->remove($c->{timeout}) if $c->{timeout};
 
   # Finish WebSocket
   my $old = $c->{tx};
@@ -347,8 +338,8 @@ sub _handle {
   elsif ($old && (my $new = $self->_upgrade($id))) {
     if (my $jar = $self->cookie_jar) { $jar->extract($old) }
     $old->client_close;
-    $self->_finish($new, $c->{cb});
-    $new->client_read($old->res->leftovers);
+    $c->{cb}->($self, $new);
+    $new->client_read($old->res->content->leftovers);
   }
 
   # Finish normal connection
@@ -356,11 +347,10 @@ sub _handle {
     $self->_remove($id, $close);
     return unless $old;
     if (my $jar = $self->cookie_jar) { $jar->extract($old) }
-    $old->client_close;
+    $old->client_close($close);
 
     # Handle redirects
-    $self->_finish($new || $old, $c->{cb}, $close)
-      unless $self->_redirect($c, $old);
+    $c->{cb}->($self, $new || $old) unless $self->_redirect($c, $old);
   }
 }
 
@@ -385,77 +375,34 @@ sub _remove {
 
   # Close connection
   my $tx = (delete($self->{connections}{$id}) || {})->{tx};
-  unless (!$close && $tx && $tx->keep_alive && !$tx->error) {
-    $self->_cache($id);
+  if ($close || !$tx || !$tx->keep_alive || $tx->error) {
+    $self->_dequeue($id);
     return $self->_loop->remove($id);
   }
 
-  # Keep connection alive
-  $self->_cache(join(':', $self->transactor->endpoint($tx)), $id)
-    unless uc $tx->req->method eq 'CONNECT' && ($tx->res->code // '') eq '200';
+  # Keep connection alive (CONNECT requests get upgraded)
+  $self->_enqueue(join(':', $self->transactor->endpoint($tx)), $id)
+    unless uc $tx->req->method eq 'CONNECT';
 }
 
 sub _redirect {
   my ($self, $c, $old) = @_;
-
-  # Follow redirect unless the maximum has been reached already
   return undef unless my $new = $self->transactor->redirect($old);
-  my $redirects = delete $c->{redirects} || 0;
-  return undef unless $redirects < $self->max_redirects;
-  my $id = $self->_start($new, delete $c->{cb});
-
-  return $self->{connections}{$id}{redirects} = $redirects + 1;
-}
-
-sub _server {
-  my ($self, $proto) = @_;
-
-  # Reuse server
-  return $self->{server} if $self->{server} && !$proto;
-
-  # Start application server
-  my $loop   = $self->_loop;
-  my $server = $self->{server}
-    = Mojo::Server::Daemon->new(ioloop => $loop, silent => 1);
-  my $port = $self->{port} ||= $loop->generate_port;
-  die "Couldn't find a free TCP port for application.\n" unless $port;
-  $self->{proto} = $proto ||= 'http';
-  $server->listen(["$proto://127.0.0.1:$port"])->start;
-  warn "-- Application server started ($proto://127.0.0.1:$port)\n" if DEBUG;
-  return $server;
+  return undef unless @{$old->redirects} < $self->max_redirects;
+  return $self->_start($new, delete $c->{cb});
 }
 
 sub _start {
   my ($self, $tx, $cb) = @_;
 
-  # Embedded server (update application if necessary)
-  my $req = $tx->req;
-  my $url = $req->url;
-  if ($self->{port} || !$url->is_abs) {
-    if (my $app = $self->app) { $self->_server->app($app) }
-    my $base = $self->app_url;
-    $url->scheme($base->scheme)->authority($base->authority)
-      unless $url->is_abs;
+  # Application server
+  my $url = $tx->req->url;
+  unless ($url->is_abs) {
+    my $base = $self->server->url;
+    $url->scheme($base->scheme)->authority($base->authority);
   }
 
-  # Proxy
-  $self->detect_proxy if $ENV{MOJO_PROXY};
-  my $proto = $url->protocol;
-  if ($self->need_proxy($url->host)) {
-
-    # HTTP proxy
-    my $http = $self->http_proxy;
-    $req->proxy($http) if $http && !defined $req->proxy && $proto eq 'http';
-
-    # HTTPS proxy
-    my $https = $self->https_proxy;
-    $req->proxy($https) if $https && !defined $req->proxy && $proto eq 'https';
-  }
-
-  # We identify ourselves and accept gzip compression
-  my $headers = $req->headers;
-  $headers->user_agent($self->name) unless $headers->user_agent;
-  $headers->accept_encoding('gzip') unless $headers->accept_encoding;
+  $self->proxy->inject($tx);
   if (my $jar = $self->cookie_jar) { $jar->inject($tx) }
 
   # Connect and add request timeout if necessary
@@ -463,7 +410,7 @@ sub _start {
   if (my $timeout = $self->request_timeout) {
     weaken $self;
     $self->{connections}{$id}{timeout} = $self->_loop->timer(
-      $timeout => sub { $self->_error($id => 'Request timeout') });
+      $timeout => sub { $self->_error($id, 'Request timeout') });
   }
 
   return $id;
@@ -511,9 +458,9 @@ Mojo::UserAgent - Non-blocking I/O HTTP and WebSocket user agent
 =head1 SYNOPSIS
 
   use Mojo::UserAgent;
-  my $ua = Mojo::UserAgent->new;
 
   # Say hello to the Unicode snowman with "Do Not Track" header
+  my $ua = Mojo::UserAgent->new;
   say $ua->get('www.☃.net?hello=there' => {DNT => 1})->res->body;
 
   # Form POST with exception handling
@@ -525,15 +472,14 @@ Mojo::UserAgent - Non-blocking I/O HTTP and WebSocket user agent
   }
 
   # Quick JSON API request with Basic authentication
-  say $ua->get('https://sri:s3cret@search.twitter.com/search.json?q=perl')
-    ->res->json('/results/0/text');
+  say $ua->get('https://sri:s3cret@example.com/search.json?q=perl')
+    ->res->json('/results/0/title');
 
   # Extract data from HTML and XML resources
-  say $ua->get('mojolicio.us')->res->dom->html->head->title->text;
+  say $ua->get('www.perl.org')->res->dom->html->head->title->text;
 
   # Scrape the latest headlines from a news site
-  say $ua->max_redirects(5)->get('www.reddit.com/r/perl/')
-    ->res->dom('p.title > a.title')->pluck('text')->shuffle;
+  say $ua->get('perlnews.org')->res->dom('h2 > a')->text->shuffle;
 
   # IPv6 PUT request with content
   my $tx
@@ -545,7 +491,7 @@ Mojo::UserAgent - Non-blocking I/O HTTP and WebSocket user agent
 
   # TLS certificate authentication and JSON POST
   my $tx = $ua->cert('tls.crt')->key('tls.key')
-    ->post('https://mojolicio.us' => json => {top => 'secret'});
+    ->post('https://example.com' => json => {top => 'secret'});
 
   # Blocking parallel requests (does not work inside a running event loop)
   my $delay = Mojo::IOLoop->delay;
@@ -572,32 +518,35 @@ Mojo::UserAgent - Non-blocking I/O HTTP and WebSocket user agent
   }
   $delay->wait unless Mojo::IOLoop->is_running;
 
-  # Non-blocking WebSocket connection sending and receiving JSON text messages
-  use Mojo::JSON 'j';
-  $ua->websocket('ws://localhost:3000/echo.json' => sub {
+  # Non-blocking WebSocket connection sending and receiving JSON messages
+  $ua->websocket('ws://example.com/echo.json' => sub {
     my ($ua, $tx) = @_;
     say 'WebSocket handshake failed!' and return unless $tx->is_websocket;
-    $tx->on(text => sub {
-      my ($tx, $bytes) = @_;
-      my $hash = j($bytes);
+    $tx->on(json => sub {
+      my ($tx, $hash) = @_;
       say "WebSocket message via JSON: $hash->{msg}";
       $tx->finish;
     });
-    $tx->send({text => j({msg => 'Hello World!'})});
+    $tx->send({json => {msg => 'Hello World!'}});
   });
   Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
 
 =head1 DESCRIPTION
 
 L<Mojo::UserAgent> is a full featured non-blocking I/O HTTP and WebSocket user
-agent, with C<IPv6>, C<TLS>, C<SNI>, C<IDNA>, C<Comet> (long polling),
-C<keep-alive>, connection pooling, timeout, cookie, multipart, proxy, C<gzip>
-compression and multiple event loop support.
+agent, with IPv6, TLS, SNI, IDNA, Comet (long polling), keep-alive, connection
+pooling, timeout, cookie, multipart, proxy, gzip compression and multiple
+event loop support.
 
-Optional modules L<EV> (4.0+), L<IO::Socket::IP> (0.16+) and
-L<IO::Socket::SSL> (1.75+) are supported transparently through
-L<Mojo::IOLoop>, and used if installed. Individual features can also be
-disabled with the MOJO_NO_IPV6 and MOJO_NO_TLS environment variables.
+All connections will be reset automatically if a new process has been forked,
+this allows multiple processes to share the same L<Mojo::UserAgent> object
+safely.
+
+For better scalability (epoll, kqueue) and to provide IPv6 as well as TLS
+support, the optional modules L<EV> (4.0+), L<IO::Socket::IP> (0.16+) and
+L<IO::Socket::SSL> (1.75+) will be used automatically by L<Mojo::IOLoop> if
+they are installed. Individual features can also be disabled with the
+MOJO_NO_IPV6 and MOJO_NO_TLS environment variables.
 
 See L<Mojolicious::Guides::Cookbook> for more.
 
@@ -613,7 +562,8 @@ the following new ones.
     ...
   });
 
-Emitted if an error occurs that can't be associated with a transaction.
+Emitted if an error occurs that can't be associated with a transaction, fatal
+if unhandled.
 
   $ua->on(error => sub {
     my ($ua, $err) = @_;
@@ -679,20 +629,6 @@ L<Mojo::UserAgent::CookieJar> object.
   # Disable cookie jar
   $ua->cookie_jar(0);
 
-=head2 http_proxy
-
-  my $proxy = $ua->http_proxy;
-  $ua       = $ua->http_proxy('http://sri:secret@127.0.0.1:8080');
-
-Proxy server to use for HTTP and WebSocket requests.
-
-=head2 https_proxy
-
-  my $proxy = $ua->https_proxy;
-  $ua       = $ua->https_proxy('http://sri:secret@127.0.0.1:8080');
-
-Proxy server to use for HTTPS and WebSocket requests.
-
 =head2 inactivity_timeout
 
   my $timeout = $ua->inactivity_timeout;
@@ -732,7 +668,7 @@ Local address to bind to.
   $ua     = $ua->max_connections(5);
 
 Maximum number of keep-alive connections that the user agent will retain
-before it starts closing the oldest cached ones, defaults to C<5>.
+before it starts closing the oldest ones, defaults to C<5>.
 
 =head2 max_redirects
 
@@ -742,19 +678,15 @@ before it starts closing the oldest cached ones, defaults to C<5>.
 Maximum number of redirects the user agent will follow before it fails,
 defaults to the value of the MOJO_MAX_REDIRECTS environment variable or C<0>.
 
-=head2 name
+=head2 proxy
 
-  my $name = $ua->name;
-  $ua      = $ua->name('Mojolicious');
+  my $proxy = $ua->proxy;
+  $ua       = $ua->proxy(Mojo::UserAgent::Proxy->new);
 
-Value for C<User-Agent> request header, defaults to C<Mojolicious (Perl)>.
+Proxy manager, defaults to a L<Mojo::UserAgent::Proxy> object.
 
-=head2 no_proxy
-
-  my $no_proxy = $ua->no_proxy;
-  $ua          = $ua->no_proxy([qw(localhost intranet.mojolicio.us)]);
-
-Domains that don't require a proxy server to be used.
+  # Detect proxy servers from environment
+  $ua->proxy->detect;
 
 =head2 request_timeout
 
@@ -770,6 +702,23 @@ indefinitely. The timeout will reset for every followed redirect.
   # Total limit of 5 seconds, of which 3 seconds may be spent connecting
   $ua->max_redirects(0)->connect_timeout(3)->request_timeout(5);
 
+=head2 server
+
+  my $server = $ua->server;
+  $ua        = $ua->server(Mojo::UserAgent::Server->new);
+
+Application server relative URLs will be processed with, defaults to a
+L<Mojo::UserAgent::Server> object.
+
+  # Introspect
+  say for @{$ua->server->app->secrets};
+
+  # Change log level
+  $ua->server->app->log->level('fatal');
+
+  # Port currently used for processing relative URLs
+  say $ua->server->url->port;
+
 =head2 transactor
 
   my $t = $ua->transactor;
@@ -777,41 +726,14 @@ indefinitely. The timeout will reset for every followed redirect.
 
 Transaction builder, defaults to a L<Mojo::UserAgent::Transactor> object.
 
+  # Change name of user agent
+  $ua->transactor->name('MyUA 1.0');
+
 =head1 METHODS
 
 L<Mojo::UserAgent> inherits all methods from L<Mojo::EventEmitter> and
 implements the following new ones.
 
-=head2 app
-
-  my $app = Mojo::UserAgent->app;
-            Mojo::UserAgent->app(MyApp->new);
-  my $app = $ua->app;
-  $ua     = $ua->app(MyApp->new);
-
-Application relative URLs will be processed with, instance specific
-applications override the global default.
-
-  # Introspect
-  say $ua->app->secret;
-
-  # Change log level
-  $ua->app->log->level('fatal');
-
-  # Change application behavior
-  $ua->app->defaults(testing => 'oh yea!');
-
-=head2 app_url
-
-  my $url = $ua->app_url;
-  my $url = $ua->app_url('http');
-  my $url = $ua->app_url('https');
-
-Get absolute L<Mojo::URL> object for C<app> and switch protocol if necessary.
-
-  # Port currently used for processing relative URLs
-  say $ua->app_url->port;
-
 =head2 build_tx
 
   my $tx = $ua->build_tx(GET => 'example.com');
@@ -831,8 +753,9 @@ L<Mojo::UserAgent::Transactor/"tx">.
 
 =head2 build_websocket_tx
 
-  my $tx = $ua->build_websocket_tx('ws://localhost:3000');
-  my $tx = $ua->build_websocket_tx('ws://localhost:3000' => {DNT => 1});
+  my $tx = $ua->build_websocket_tx('ws://example.com');
+  my $tx = $ua->build_websocket_tx(
+    'ws://example.com' => {DNT => 1} => ['v1.proto']);
 
 Generate L<Mojo::Transaction::HTTP> object with
 L<Mojo::UserAgent::Transactor/"websocket">.
@@ -846,7 +769,7 @@ L<Mojo::UserAgent::Transactor/"websocket">.
   my $tx = $ua->delete(
     'http://example.com' => {DNT => 1} => json => {a => 'b'});
 
-Perform blocking HTTP C<DELETE> request and return resulting
+Perform blocking DELETE request and return resulting
 L<Mojo::Transaction::HTTP> object, takes the same arguments as
 L<Mojo::UserAgent::Transactor/"tx"> (except for the method). You can also
 append a callback to perform requests non-blocking.
@@ -857,14 +780,6 @@ append a callback to perform requests non-blocking.
   });
   Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
 
-=head2 detect_proxy
-
-  $ua = $ua->detect_proxy;
-
-Check environment variables HTTP_PROXY, http_proxy, HTTPS_PROXY, https_proxy,
-NO_PROXY and no_proxy for proxy information. Automatic proxy detection can be
-enabled with the MOJO_PROXY environment variable.
-
 =head2 get
 
   my $tx = $ua->get('example.com');
@@ -872,10 +787,10 @@ enabled with the MOJO_PROXY environment variable.
   my $tx = $ua->get('http://example.com' => {DNT => 1} => form => {a => 'b'});
   my $tx = $ua->get('http://example.com' => {DNT => 1} => json => {a => 'b'});
 
-Perform blocking HTTP C<GET> request and return resulting
-L<Mojo::Transaction::HTTP> object, takes the same arguments as
-L<Mojo::UserAgent::Transactor/"tx"> (except for the method). You can also
-append a callback to perform requests non-blocking.
+Perform blocking GET request and return resulting L<Mojo::Transaction::HTTP>
+object, takes the same arguments as L<Mojo::UserAgent::Transactor/"tx">
+(except for the method). You can also append a callback to perform requests
+non-blocking.
 
   $ua->get('http://example.com' => sub {
     my ($ua, $tx) = @_;
@@ -892,10 +807,10 @@ append a callback to perform requests non-blocking.
   my $tx = $ua->head(
     'http://example.com' => {DNT => 1} => json => {a => 'b'});
 
-Perform blocking HTTP C<HEAD> request and return resulting
-L<Mojo::Transaction::HTTP> object, takes the same arguments as
-L<Mojo::UserAgent::Transactor/"tx"> (except for the method). You can also
-append a callback to perform requests non-blocking.
+Perform blocking HEAD request and return resulting L<Mojo::Transaction::HTTP>
+object, takes the same arguments as L<Mojo::UserAgent::Transactor/"tx">
+(except for the method). You can also append a callback to perform requests
+non-blocking.
 
   $ua->head('http://example.com' => sub {
     my ($ua, $tx) = @_;
@@ -903,12 +818,6 @@ append a callback to perform requests non-blocking.
   });
   Mojo::IOLoop->start unless Mojo::IOLoop->is_running;
 
-=head2 need_proxy
-
-  my $success = $ua->need_proxy('intranet.mojolicio.us');
-
-Check if request for domain would use a proxy server.
-
 =head2 options
 
   my $tx = $ua->options('example.com');
@@ -918,7 +827,7 @@ Check if request for domain would use a proxy server.
   my $tx = $ua->options(
     'http://example.com' => {DNT => 1} => json => {a => 'b'});
 
-Perform blocking HTTP C<OPTIONS> request and return resulting
+Perform blocking OPTIONS request and return resulting
 L<Mojo::Transaction::HTTP> object, takes the same arguments as
 L<Mojo::UserAgent::Transactor/"tx"> (except for the method). You can also
 append a callback to perform requests non-blocking.
@@ -938,10 +847,10 @@ append a callback to perform requests non-blocking.
   my $tx = $ua->patch(
     'http://example.com' => {DNT => 1} => json => {a => 'b'});
 
-Perform blocking HTTP C<PATCH> request and return resulting
-L<Mojo::Transaction::HTTP> object, takes the same arguments as
-L<Mojo::UserAgent::Transactor/"tx"> (except for the method). You can also
-append a callback to perform requests non-blocking.
+Perform blocking PATCH request and return resulting L<Mojo::Transaction::HTTP>
+object, takes the same arguments as L<Mojo::UserAgent::Transactor/"tx">
+(except for the method). You can also append a callback to perform requests
+non-blocking.
 
   $ua->patch('http://example.com' => sub {
     my ($ua, $tx) = @_;
@@ -958,10 +867,10 @@ append a callback to perform requests non-blocking.
   my $tx = $ua->post(
     'http://example.com' => {DNT => 1} => json => {a => 'b'});
 
-Perform blocking HTTP C<POST> request and return resulting
-L<Mojo::Transaction::HTTP> object, takes the same arguments as
-L<Mojo::UserAgent::Transactor/"tx"> (except for the method). You can also
-append a callback to perform requests non-blocking.
+Perform blocking POST request and return resulting L<Mojo::Transaction::HTTP>
+object, takes the same arguments as L<Mojo::UserAgent::Transactor/"tx">
+(except for the method). You can also append a callback to perform requests
+non-blocking.
 
   $ua->post('http://example.com' => sub {
     my ($ua, $tx) = @_;
@@ -976,10 +885,10 @@ append a callback to perform requests non-blocking.
   my $tx = $ua->put('http://example.com' => {DNT => 1} => form => {a => 'b'});
   my $tx = $ua->put('http://example.com' => {DNT => 1} => json => {a => 'b'});
 
-Perform blocking HTTP C<PUT> request and return resulting
-L<Mojo::Transaction::HTTP> object, takes the same arguments as
-L<Mojo::UserAgent::Transactor/"tx"> (except for the method). You can also
-append a callback to perform requests non-blocking.
+Perform blocking PUT request and return resulting L<Mojo::Transaction::HTTP>
+object, takes the same arguments as L<Mojo::UserAgent::Transactor/"tx">
+(except for the method). You can also append a callback to perform requests
+non-blocking.
 
   $ua->put('http://example.com' => sub {
     my ($ua, $tx) = @_;
@@ -1003,15 +912,16 @@ non-blocking.
 
 =head2 websocket
 
-  $ua->websocket('ws://localhost:3000' => sub {...});
-  $ua->websocket('ws://localhost:3000' => {DNT => 1} => sub {...});
+  $ua->websocket('ws://example.com' => sub {...});
+  $ua->websocket(
+    'ws://example.com' => {DNT => 1} => ['v1.proto'] => sub {...});
 
 Open a non-blocking WebSocket connection with transparent handshake, takes the
 same arguments as L<Mojo::UserAgent::Transactor/"websocket">. The callback
 will receive either a L<Mojo::Transaction::WebSocket> or
 L<Mojo::Transaction::HTTP> object.
 
-  $ua->websocket('ws://localhost:3000/echo' => sub {
+  $ua->websocket('ws://example.com/echo' => sub {
     my ($ua, $tx) = @_;
     say 'WebSocket handshake failed!' and return unless $tx->is_websocket;
     $tx->on(finish => sub {
