@@ -13,7 +13,9 @@ use Mojolicious::Routes;
 use Mojolicious::Sessions;
 use Mojolicious::Static;
 use Mojolicious::Types;
+use Mojolicious::Validator;
 use Scalar::Util qw(blessed weaken);
+use Time::HiRes 'gettimeofday';
 
 has commands => sub {
   my $commands = Mojolicious::Commands->new(app => shift);
@@ -21,26 +23,27 @@ has commands => sub {
   return $commands;
 };
 has controller_class => 'Mojolicious::Controller';
-has mode => sub { $ENV{MOJO_MODE} || 'development' };
+has mode => sub { $ENV{MOJO_MODE} || $ENV{PLACK_ENV} || 'development' };
 has moniker  => sub { decamelize ref shift };
 has plugins  => sub { Mojolicious::Plugins->new };
 has renderer => sub { Mojolicious::Renderer->new };
 has routes   => sub { Mojolicious::Routes->new };
-has secret   => sub {
+has secrets  => sub {
   my $self = shift;
 
   # Warn developers about insecure default
   $self->log->debug('Your secret passphrase needs to be changed!!!');
 
-  # Default to application name
-  return ref $self;
+  # Default to moniker
+  return [$self->moniker];
 };
-has sessions => sub { Mojolicious::Sessions->new };
-has static   => sub { Mojolicious::Static->new };
-has types    => sub { Mojolicious::Types->new };
+has sessions  => sub { Mojolicious::Sessions->new };
+has static    => sub { Mojolicious::Static->new };
+has types     => sub { Mojolicious::Types->new };
+has validator => sub { Mojolicious::Validator->new };
 
-our $CODENAME = 'Rainbow';
-our $VERSION  = '3.97';
+our $CODENAME = 'Top Hat';
+our $VERSION  = '4.63';
 
 sub AUTOLOAD {
   my $self = shift;
@@ -68,19 +71,19 @@ sub new {
   my $r = $self->routes->namespaces([ref $self]);
 
   # Hide controller attributes/methods and "handler"
-  $r->hide(qw(AUTOLOAD DESTROY app cookie finish flash handler on param));
-  $r->hide(qw(redirect_to render render_data render_exception render_json));
-  $r->hide(qw(render_not_found render_partial render_static render_text));
-  $r->hide(qw(rendered req res respond_to send session signed_cookie stash));
-  $r->hide(qw(tx ua url_for write write_chunk));
+  $r->hide(qw(app continue cookie finish flash handler match on param));
+  $r->hide(qw(redirect_to render render_exception render_later render_maybe));
+  $r->hide(qw(render_not_found render_static rendered req res respond_to));
+  $r->hide(qw(send session signed_cookie stash tx url_for validation write));
+  $r->hide(qw(write_chunk));
 
   # Check if we have a log directory
   my $mode = $self->mode;
   $self->log->path($home->rel_file("log/$mode.log"))
     if -w $home->rel_file('log');
 
-  $self->plugin($_) for qw(HeaderCondition DefaultHelpers TagHelpers);
-  $self->plugin($_) for qw(EPLRenderer EPRenderer RequestTimer PoweredBy);
+  $self->plugin($_)
+    for qw(HeaderCondition DefaultHelpers TagHelpers EPLRenderer EPRenderer);
 
   # Exception handling should be first in chain
   $self->hook(around_dispatch => \&_exception);
@@ -88,9 +91,7 @@ sub new {
   # Reduced log output outside of development mode
   $self->log->level('info') unless $mode eq 'development';
 
-  # Run mode before startup
-  if (my $sub = $self->can("${mode}_mode")) { $self->$sub(@_) }
-  $self->startup(@_);
+  $self->startup;
 
   return $self;
 }
@@ -109,7 +110,7 @@ sub dispatch {
 
   # Prepare transaction
   my $tx = $c->tx;
-  $c->res->code(undef) if $tx->is_websocket;
+  $tx->res->code(undef) if $tx->is_websocket;
   $self->sessions->load($c);
   my $plugins = $self->plugins->emit_hook(before_dispatch => $c);
 
@@ -117,11 +118,14 @@ sub dispatch {
   $self->static->dispatch($c) and $plugins->emit_hook(after_static => $c)
     unless $tx->res->code;
 
-  # DEPRECATED in Rainbow!
-  if ($plugins->has_subscribers('after_static_dispatch')) {
-    deprecated
-      'after_static_dispatch hook is DEPRECATED in favor of before_routes'
-      and $plugins->emit_hook_reverse(after_static_dispatch => $c);
+  # Start timer (ignore static files)
+  my $stash = $c->stash;
+  unless ($stash->{'mojo.static'} || $stash->{'mojo.started'}) {
+    my $req    = $c->req;
+    my $method = $req->method;
+    my $path   = $req->url->path->to_abs_string;
+    $self->log->debug(qq{$method "$path".});
+    $stash->{'mojo.started'} = [gettimeofday];
   }
 
   # Routes
@@ -139,7 +143,7 @@ sub handler {
   # Embedded application
   my $stash = {};
   if (my $sub = $tx->can('stash')) { ($stash, $tx) = ($tx->$sub, $tx->tx) }
-  $stash->{'mojo.secret'} //= $self->secret;
+  $stash->{'mojo.secrets'} //= $self->secrets;
 
   # Build default controller
   my $defaults = $self->defaults;
@@ -150,19 +154,16 @@ sub handler {
 
   # Dispatcher has to be last in the chain
   ++$self->{dispatch}
+    and $self->hook(around_action   => sub { $_[2]->($_[1]) })
     and $self->hook(around_dispatch => sub { $_[1]->app->dispatch($_[1]) })
     unless $self->{dispatch};
 
   # Process with chain
-  unless (eval { $self->plugins->emit_chain(around_dispatch => $c) }) {
-    $self->log->fatal("Processing request failed: $@");
-    $tx->res->code(500);
-    $tx->resume;
-  }
+  $self->plugins->emit_chain(around_dispatch => $c);
 
   # Delayed response
   $self->log->debug('Nothing has been rendered, expecting delayed response.')
-    unless $stash->{'mojo.rendered'} || $tx->is_writing;
+    unless $tx->is_writing;
 }
 
 sub helper {
@@ -180,6 +181,16 @@ sub plugin {
   $self->plugins->register_plugin(shift, $self, @_);
 }
 
+# DEPRECATED in Top Hat!
+sub secret {
+  deprecated
+    'Mojolicious::secret is DEPRECATED in favor of Mojolicious::secrets';
+  my $self = shift;
+  return $self->secrets->[0] unless @_;
+  $self->secrets->[0] = shift;
+  return $self;
+}
+
 sub start { shift->commands->run(@_ ? @_ : @ARGV) }
 
 sub startup { }
@@ -193,6 +204,8 @@ sub _exception {
 
 1;
 
+=encoding utf8
+
 =head1 NAME
 
 Mojolicious - Real-time web framework
@@ -223,6 +236,131 @@ Mojolicious - Real-time web framework
 
 Take a look at our excellent documentation in L<Mojolicious::Guides>!
 
+=head1 HOOKS
+
+L<Mojolicious> will emit the following hooks in the listed order.
+
+=head2 after_build_tx
+
+Emitted right after the transaction is built and before the HTTP request gets
+parsed.
+
+  $app->hook(after_build_tx => sub {
+    my ($tx, $app) = @_;
+    ...
+  });
+
+This is a very powerful hook and should not be used lightly, it makes some
+rather advanced features such as upload progress bars possible. Note that this
+hook will not work for embedded applications. (Passed the transaction and
+application object)
+
+=head2 before_dispatch
+
+Emitted right before the static file server and router start their work.
+
+  $app->hook(before_dispatch => sub {
+    my $c = shift;
+    ...
+  });
+
+Very useful for rewriting incoming requests and other preprocessing tasks.
+(Passed the default controller object)
+
+=head2 after_static
+
+Emitted after a static file response has been generated by the static file
+server.
+
+  $app->hook(after_static => sub {
+    my $c = shift;
+    ...
+  });
+
+Mostly used for post-processing static file responses. (Passed the default
+controller object)
+
+=head2 before_routes
+
+Emitted after the static file server determined if a static file should be
+served and before the router starts its work.
+
+  $app->hook(before_routes => sub {
+    my $c = shift;
+    ...
+  });
+
+Mostly used for custom dispatchers and collecting metrics. (Passed the default
+controller object)
+
+=head2 around_action
+
+Emitted right before an action gets invoked and wraps around it, so you have
+to manually forward to the next hook if you want to continue the chain.
+Default action dispatching is the last hook in the chain, yours will run
+before it.
+
+  $app->hook(around_action => sub {
+    my ($next, $c, $action, $last) = @_;
+    ...
+    return $next->();
+  });
+
+This is a very powerful hook and should not be used lightly, it allows you for
+example to pass additional arguments to actions or handle return values
+differently. (Passed a callback leading to the next hook, the current
+controller object, the action callback and a flag indicating if this action is
+an endpoint)
+
+=head2 after_render
+
+Emitted after content has been generated by the renderer that is not partial.
+Note that this hook can trigger out of order due to its dynamic nature, and
+with embedded applications will only work for the application that is
+rendering.
+
+  $app->hook(after_render => sub {
+    my ($c, $output, $format) = @_;
+    ...
+  });
+
+Mostly used for post-processing dynamically generated content. (Passed the
+current controller object, a reference to the content and the format)
+
+=head2 after_dispatch
+
+Emitted in reverse order after a response has been rendered. Note that this
+hook can trigger out of order due to its dynamic nature, and with embedded
+applications will only work for the application that is rendering.
+
+  $app->hook(after_dispatch => sub {
+    my $c = shift;
+    ...
+  });
+
+Useful for rewriting outgoing responses and other post-processing tasks.
+(Passed the current controller object)
+
+=head2 around_dispatch
+
+Emitted right before the L</"before_dispatch"> hook and wraps around the whole
+dispatch process, so you have to manually forward to the next hook if you want
+to continue the chain. Default exception handling with
+L<Mojolicious::Controller/"render_exception"> is the first hook in the chain
+and a call to L</"dispatch"> the last, yours will be in between.
+
+  $app->hook(around_dispatch => sub {
+    my ($next, $c) = @_;
+    ...
+    $next->();
+    ...
+  });
+
+This is a very powerful hook and should not be used lightly, it allows you for
+example to customize application wide exception handling, consider it the
+sledgehammer in your toolbox. (Passed a callback leading to the next hook and
+the default controller object)
+
 =head1 ATTRIBUTES
 
 L<Mojolicious> inherits all attributes from L<Mojo> and implements the
@@ -252,24 +390,11 @@ L<Mojolicious::Controller>.
   my $mode = $app->mode;
   $app     = $app->mode('production');
 
-The operating mode for your application, defaults to the value of the
-MOJO_MODE environment variable or C<development>. You can also add per
-mode logic to your application by defining methods named C<${mode}_mode> in
-the application class, which will be called right before C<startup>.
-
-  sub development_mode {
-    my $self = shift;
-    ...
-  }
-
-  sub production_mode {
-    my $self = shift;
-    ...
-  }
-
-Right before calling C<startup> and mode specific methods, L<Mojolicious>
-will pick up the current mode, name the log file after it and raise the log
-level from C<debug> to C<info> if it has a value other than C<development>.
+The operating mode for your application, defaults to a value from the
+MOJO_MODE and PLACK_ENV environment variables or C<development>. Right before
+calling L</"startup">, L<Mojolicious> will pick up the current mode, name the
+log file after it and raise the log level from C<debug> to C<info> if it has a
+value other than C<development>.
 
 =head2 moniker
 
@@ -286,7 +411,7 @@ L<Mojo::Util/"decamelize">.
   $app        = $app->plugins(Mojolicious::Plugins->new);
 
 The plugin manager, defaults to a L<Mojolicious::Plugins> object. See the
-C<plugin> method below if you want to load a plugin.
+L</"plugin"> method below if you want to load a plugin.
 
   # Add another namespace to load plugins from
   push @{$app->plugins->namespaces}, 'MyApp::Plugin';
@@ -323,15 +448,22 @@ startup method to define the url endpoints for your application.
   # Add another namespace to load controllers from
   push @{$app->routes->namespaces}, 'MyApp::Controller';
 
-=head2 secret
+=head2 secrets
 
-  my $secret = $app->secret;
-  $app       = $app->secret('passw0rd');
+  my $secrets = $app->secrets;
+  $app        = $app->secrets(['passw0rd']);
 
-A secret passphrase used for signed cookies and the like, defaults to the
-application name which is not very secure, so you should change it!!! As long
-as you are using the insecure default there will be debug messages in the log
-file reminding you to change your passphrase.
+Secret passphrases used for signed cookies and the like, defaults to the
+L</"moniker"> of this application, which is not very secure, so you should
+change it!!! As long as you are using the insecure default there will be debug
+messages in the log file reminding you to change your passphrase. Only the
+first passphrase is used to create new signatures, but all of them for
+verification. So you can increase security without invalidating all your
+signed cookies by rotating passphrases, just add new ones to the front and
+remove old ones from the back.
+
+  # Rotate passphrases
+  $app->secrets(['new_passw0rd', 'old_passw0rd', 'very_old_passw0rd']);
 
 =head2 sessions
 
@@ -371,6 +503,13 @@ L<Mojolicious::Types> object.
   # Add custom MIME type
   $app->types->type(twt => 'text/tweet');
 
+=head2 validator
+
+  my $validator = $app->validator;
+  $app          = $app->validator(Mojolicious::Validator->new);
+
+Validate parameters, defaults to a L<Mojolicious::Validator> object.
+
 =head1 METHODS
 
 L<Mojolicious> inherits all methods from L<Mojo> and implements the following
@@ -381,9 +520,10 @@ new ones.
   my $app = Mojolicious->new;
 
 Construct a new L<Mojolicious> application, calling C<${mode}_mode> and
-C<startup> in the process. Will automatically detect your home directory and
-set up logging based on your current operating mode. Also sets up the
-renderer, static file server and a default set of plugins.
+L</"startup"> in the process. Will automatically detect your home directory
+and set up logging based on your current operating mode. Also sets up the
+renderer, static file server, a default set of plugins and an
+L</"around_dispatch"> hook with the default exception handling.
 
 =head2 build_tx
 
@@ -394,33 +534,32 @@ object.
 
 =head2 defaults
 
-  my $defaults = $app->defaults;
-  my $foo      = $app->defaults('foo');
-  $app         = $app->defaults({foo => 'bar'});
-  $app         = $app->defaults(foo => 'bar');
+  my $hash = $app->defaults;
+  my $foo  = $app->defaults('foo');
+  $app     = $app->defaults({foo => 'bar'});
+  $app     = $app->defaults(foo => 'bar');
 
 Default values for L<Mojolicious::Controller/"stash">, assigned for every new
 request.
 
-  # Manipulate defaults
-  $app->defaults->{foo} = 'bar';
-  my $foo = $app->defaults->{foo};
-  delete $app->defaults->{foo};
+  # Remove value
+  my $foo = delete $app->defaults->{foo};
 
 =head2 dispatch
 
   $app->dispatch(Mojolicious::Controller->new);
 
-The heart of every Mojolicious application, calls the C<static> and C<routes>
-dispatchers for every request and passes them a L<Mojolicious::Controller>
-object.
+The heart of every L<Mojolicious> application, calls the L</"static"> and
+L</"routes"> dispatchers for every request and passes them a
+L<Mojolicious::Controller> object.
 
 =head2 handler
 
   $app->handler(Mojo::Transaction::HTTP->new);
   $app->handler(Mojolicious::Controller->new);
 
-Sets up the default controller and calls process for every request.
+Sets up the default controller and emits the L</"around_dispatch"> hook for
+every request.
 
 =head2 helper
 
@@ -445,123 +584,15 @@ and the application object, as well as a function in C<ep> templates.
   $app->hook(after_dispatch => sub {...});
 
 Extend L<Mojolicious> with hooks, which allow code to be shared with all
-requests indiscriminately.
+requests indiscriminately, for a full list of available hooks see L</"HOOKS">.
 
   # Dispatchers will not run if there's already a response code defined
   $app->hook(before_dispatch => sub {
     my $c = shift;
-    $c->render(text => 'Skipped dispatchers!')
+    $c->render(text => 'Skipped static file server and router!')
       if $c->req->url->path->to_route =~ /do_not_dispatch/;
   });
 
-These hooks are currently available and are emitted in the listed order:
-
-=over 2
-
-=item after_build_tx
-
-Emitted right after the transaction is built and before the HTTP request gets
-parsed.
-
-  $app->hook(after_build_tx => sub {
-    my ($tx, $app) = @_;
-    ...
-  });
-
-This is a very powerful hook and should not be used lightly, it makes some
-rather advanced features such as upload progress bars possible. Note that this
-hook will not work for embedded applications. (Passed the transaction and
-application object)
-
-=item before_dispatch
-
-Emitted right before the static file server and router start their work.
-
-  $app->hook(before_dispatch => sub {
-    my $c = shift;
-    ...
-  });
-
-Very useful for rewriting incoming requests and other preprocessing tasks.
-(Passed the default controller object)
-
-=item after_static
-
-Emitted after a static file response has been generated by the static file
-server.
-
-  $app->hook(after_static => sub {
-    my $c = shift;
-    ...
-  });
-
-Mostly used for post-processing static file responses. (Passed the default
-controller object)
-
-=item before_routes
-
-Emitted after the static file server determined if a static file should be
-served and before the router starts its work.
-
-  $app->hook(before_routes => sub {
-    my $c = shift;
-    ...
-  });
-
-Mostly used for custom dispatchers and collecting metrics. (Passed the default
-controller object)
-
-=item after_render
-
-Emitted after content has been generated by the renderer that is not partial.
-Note that this hook can trigger out of order due to its dynamic nature, and
-with embedded applications will only work for the application that is
-rendering.
-
-  $app->hook(after_render => sub {
-    my ($c, $output, $format) = @_;
-    ...
-  });
-
-Mostly used for post-processing dynamically generated content. (Passed the
-current controller object, a reference to the content and the format)
-
-=item after_dispatch
-
-Emitted in reverse order after a response has been rendered. Note that this
-hook can trigger out of order due to its dynamic nature, and with embedded
-applications will only work for the application that is rendering.
-
-  $app->hook(after_dispatch => sub {
-    my $c = shift;
-    ...
-  });
-
-Useful for rewriting outgoing responses and other post-processing tasks.
-(Passed the current controller object)
-
-=item around_dispatch
-
-Emitted right before the C<before_dispatch> hook and wraps around the whole
-dispatch process, so you have to manually forward to the next hook if you want
-to continue the chain. Default exception handling with
-L<Mojolicious::Controller/"render_exception"> is the first hook in the chain
-and a call to C<dispatch> the last, yours will be in between.
-
-  $app->hook(around_dispatch => sub {
-    my ($next, $c) = @_;
-    ...
-    $next->();
-    ...
-  });
-
-This is a very powerful hook and should not be used lightly, it allows you to
-customize application wide exception handling for example, consider it the
-sledgehammer in your toolbox. (Passed a callback leading to the next hook and
-the default controller object)
-
-=back
-
 =head2 plugin
 
   $app->plugin('some_thing');
@@ -620,8 +651,8 @@ that have been bundled for internal use.
 
   Copyright (C) 2010-2013, Sebastian Riedel.
 
-Licensed under the CC-SA License, Version 3.0
-L<http://creativecommons.org/licenses/by-sa/3.0>.
+Licensed under the CC-SA License, Version 4.0
+L<http://creativecommons.org/licenses/by-sa/4.0>.
 
 =head2 jQuery
 
@@ -641,6 +672,8 @@ L<http://www.apache.org/licenses/LICENSE-2.0>.
 Every major release of L<Mojolicious> has a code name, these are the ones that
 have been used in the past.
 
+4.0, C<Top Hat> (u1F3A9)
+
 3.0, C<Rainbow> (u1F308)
 
 2.0, C<Leaf Fluttering In Wind> (u1F343)
@@ -672,7 +705,7 @@ Sebastian Riedel, C<sri@cpan.org>
 
 Current members of the core team in alphabetical order:
 
-=over 4
+=over 2
 
 Abhijit Menon-Sen, C<ams@cpan.org>
 
@@ -696,6 +729,8 @@ Adriano Ferreira
 
 Al Newkirk
 
+Alex Efros
+
 Alex Salimon
 
 Alexey Likhatskiy
@@ -732,6 +767,8 @@ Breno G. de Oliveira
 
 Brian Duggan
 
+Brian Medley
+
 Burak Gursoy
 
 Ch Lamprecht
@@ -778,10 +815,14 @@ Henry Tang
 
 Hideki Yamamura
 
+Ian Goodacre
+
 Ilya Chesnokov
 
 James Duncan
 
+Jan Henning Thorsen
+
 Jan Jona Javorsek
 
 Jan Schmidt
@@ -796,6 +837,8 @@ John Kingsley
 
 Jonathan Yu
 
+Josh Leder
+
 Kazuhiro Shibuya
 
 Kevin Old
@@ -844,6 +887,8 @@ Paul Evans
 
 Paul Tomlin
 
+Pavel Shaydo
+
 Pedro Melo
 
 Peter Edwards
